



#include "MiscCommandline.h"
#include "base58.h"
#include "commandLine.h"
#include "miniPriv.h"
#include "extraInfo.h"
#include "privKey.h"
#include "mnemonic.h"
#include "GenerationParameters.h"
#include "GenerationMethods.h"
#include "language.h"
#include "printPS.h"
#include "bitgenVersion.h"
#include "FileUtil.h"
#include "bitgenConfig.h"
#include "keyDerivationFunction.h"
#include "TransactionCommandline.h"


#include <iostream>
#include <list>
#include <stdlib.h>
#include <stdio.h>



int parseNumTextWords(const std::string& numWordsStr, std::string& wallet)
{
	if(numWordsStr == "jaxx")
	{
		wallet = "Jaxx";
		return 12;
	}
	else if(numWordsStr == "coinomi")
	{
		wallet = "Coinomi";		
		return 18;
	}
	else if(numWordsStr == "exodus")
	{
		wallet = "Exodus";		
		return 12;
	}
	else if(numWordsStr == "mycelium")
	{
		wallet = "Mycelium";
		return 12;
	}
	else if(numWordsStr == "multibit")
	{
		wallet = "Multibit HD";
		return 12;
	}
	
	
	const int numWords = atoi(numWordsStr.c_str());
	return numWords;
}






void listMnemonicSize(const std::vector<std::string>& args, const std::string& thetime)
{
	if(args.size() < 1)
	{
		std::cout << "Usage: bitgen listMnemonic" << std::endl;
		return;
	}	

	std::cout << "WALLET      Default Minsize  MaxSize      (TextWords)" << std::endl;
	std::cout << "=====================================================" << std::endl;	
	std::cout << "Coinomi:    18      12       24" << std::endl;
	std::cout << "Mycelium:   12                 " << std::endl;
	std::cout << "Exodus      12                 " << std::endl;
	std::cout << "Jaxx        12                 " << std::endl;
	std::cout << "MultibitHD  12      12       24" << std::endl;
	
	
	std::cout << std::endl;
}






void getMnemonicInformation(const std::vector<std::string>& args, const std::string& thetime)
{
	if(args.size() < 2)
	{
		std::cout << "Usage: bitgen mnemonicinfo MNEMONIC ..." << std::endl;
		return;
	}	

	std::string mnemonicStr(args[1]);
	std::cout << "Mnemonic: " << mnemonicStr << std::endl;
	
	const Mnemonic mnemonic(mnemonicStr); 
	
	const std::string privKey = mnemonic.getAsNumber();
	
	std::cout << "Private data (without checksum):     " << hexString(privKey) << std::endl;
	
	const ExtendedPrivateKey extPk = calculateMasterKey(privKey);	
}









void getCppString(const std::vector<std::string>& args)
{	
	if(args.size() < 2)
	{
		std::cout << "Usage: bitgen cppstr hextransaction" << std::endl;	
		return;
	}	
	
	std::cout << std::endl;
	
	const std::string hexStr(args[1]);
	
	
	//Determine if filename or hex string	
	//const std::string trans
	const BinaryTransaction trans = TransactionCommandline::getTrans(hexStr);
	
	const std::string hex = trans.asHex();
		
	std::cout << "Orig trans: " << std::endl;
	
	for(int i = 0 ; i < hex.size() ; i++)
	{
		if(i % 79 == 0)
		{
			std::cout << "\\\n";			
		}
				
		const unsigned char c = hex[i];		
		std::cout << c;
	}
	std::cout << "\n";				
}






void showAscii(const std::vector<std::string>& args)
{	
	if(args.size() < 2)
	{
		std::cout << "Usage: bitgen showascii hextransaction" << std::endl;	
		return;
	}	

	const std::string hexStr(args[1]);

	const std::string trans = convertFromHex(hexStr);

	for(int i = 0 ; i < trans.size() ; i++)
	{
		const unsigned char c = trans[i];
		
		if(c >= 32 &&  c <= 128)
		{
			std::cout << c;
		}
		else
		{
			std::cout << "_";
		}
	}
	
	std::cout << std::endl;
}








#ifndef NO_BRAINWALLET_GENERATION

void testArgon(const std::string& passphrase, const std::string& salt,
	const std::string& hashNumStr)
{

	std::cout << "Testing argon... " << std::flush;
	const std::string kdfStr = bitgenArgon2Kdf(passphrase, salt);
	const BigInt<512> hashNumComputed(kdfStr, 'c');
	
	const BigInt<512> hashNum(BigInt<512>::fromHex(hashNumStr));
	if(hashNum != hashNumComputed)
	{
		std::cout << std::endl;
		std::cout << "Error, computed kdfStr: " << hexString(kdfStr) << std::endl;
		
		throw std::string("Argon error");
	}

	std::cout << "OK" << std::endl;
	
}


void brainwalletTests()
{	
	const std::string passphrase("1234567890abcdefghijklmnopq");
	const std::string salt("someone@email.com");
	const std::string hashNumStr("9A5D21E6D7F1D3D1C5A7FBE2547F16ABF2DAE34379FA6309F9672491E2D83276");
	testArgon(passphrase, salt, hashNumStr);
	
	
	const std::string passphrase2("abc");
	const std::string salt2("this@abc.org");
	const std::string hashNumStr2("BD8BBBA3F95A59CA5E3100B4EFC488A9395A253F1366DFBCD84632A2F204C23D");	
	testArgon(passphrase2, salt2, hashNumStr2);
	
	
	const std::string passphrase3("");
	const std::string salt3("");
	const std::string hashNumStr3("93687FFD2E4E6A6A4E09188D6E1CCD3D981981597FE95237231A17DB6991E4F7");
	testArgon(passphrase3, salt3, hashNumStr3);
	
}
#endif









void printIntSize()
{
	std::cout << "sizeof(int)        : " << sizeof(int) << " gives " << sizeof(int)*8 << " bits" << std::endl;
	std::cout << "sizeof(long)       : " << sizeof(long) << " gives " << sizeof(long)*8 << " bits" << std::endl;
	std::cout << "sizeof(long long)  : " << sizeof(long long) << " gives " << sizeof(long long)*8 << " bits" << std::endl;
#if __x86_64__ || __ppc64__	
	std::cout << "sizeof(__uint128_t): " << sizeof(__uint128_t) << " gives " << sizeof(__uint128_t)*8 << " bits" << std::endl;	
#endif
}








void uncompressPublicKey(const std::vector<std::string>& args, /* int argc, char* argv[], */ const std::string& thetime)
{
	if(args.size() < 2) //argc < 3)
	{
		std::cout << "Usage: bitgen uncompress COMPRESSEDPUBKEY" << std::endl;
		return;			
	}

	const std::string pubkeyStr(args[1]); //argv[2]);
	
	if(pubkeyStr.size() != 66)
	{
		std::cout << "Got key with size: " << pubkeyStr.size() << ", but need 66 chars" << std::endl;
		throw std::string("Error, compressed key size must be 66 charcters");
	}

	const PubKey pubKey = PubKey::fromHex(pubkeyStr);
	
	const std::string uncompStr = hexString(pubKey.point.getRaw(false));
	
	std::cout << "Uncompressed: " << uncompStr << std::endl;
}




void compressPublicKey(const std::vector<std::string>& args, /* int argc, char* argv[], */ const std::string& thetime)
{
	if(args.size() < 2) //argc < 3)
	{
		std::cout << "Usage: bitgen compress VERBOSEPUBKEY" << std::endl;
		return;			
	}

	const std::string pubkeyStr(args[1]); //argv[2]);
	std::cout << "Will convert: " << pubkeyStr << std::endl;
	
	if(pubkeyStr.size() != 130)
	{
		std::cout << "Got key with size: " << pubkeyStr.size() << ", but need 130 chars" << std::endl;
		throw std::string("Error, verbose key size must be 130 charcters");
	}
	
	const PubKey pubKey = PubKey::fromHex(pubkeyStr);
	
	const std::string compStr = hexString(pubKey.point.getRaw(true));
	
	std::cout << "Compressed: " << compStr << std::endl;
}





void generatePaymentRequest(int argc, char* argv[], const std::string& thetime)
{
	if(argc < 3)
	{
		std::cout << "Usage: bitgen invoice coinAddress amount [label] [message] [LastDate]" << std::endl;
		return;			
	}
	
	const std::string adr(argv[2]);
	const PubAddress myPub(adr);

	std::string amount;
	if(argc > 3)
	{
		const std::string amountGiven(argv[3]); 
		//Make sure the value is a non-zero number
		const double value = atof(amountGiven.c_str());
		if(value == 0.0)
		{
			throw std::string("Incorrect or zero sum given");
		}
		
		std::cout << "Using amount: " << value << std::endl;
		char str[128];
		sprintf(str, "%0f", value);
		amount = str;
	}


	std::string label;
	if(argc > 4)
	{
		label = argv[4];
	}

	std::string message;
	if(argc > 5)
	{
		message = argv[5];
	}

	std::string lastDate;
	if(argc > 6)
	{
		lastDate = argv[6];
	}
		
	const std::list<std::string> not_used_printExtra;

	const GenerationInfoBase infoBase(thetime, getNodeName(), applicationName, versionNumber);

	GenerationParameters params;
	params.setAccountName(label);
	
	std::list<ExtraInfo> generationExtra;

	const int newNum = GenerationMethods::getNextNumberIfConfigured(params);
	
	
	const GenerationInfoCore infoCore(infoBase, newNum, params);
	const GenerationInfo info(infoCore, generationExtra, "theGenMethod");
	const AddressGenerationInfo addrInfo(myPub.toString(), "", "", info);


	const Language lang = getLanguage();
	const PostscriptGenerator postscriptGenerator(info, versionNumber, applicationName, lang);
	const PublicPostscriptInfo pubInfo(addrInfo);
	
	const std::string psPublic = postscriptGenerator.generateInvoice(pubInfo, amount, message, lastDate);
	
	const std::string pubFilename = "num_" + intToString(info.getNewNum()) + "_" + pubInfo.pubkey.substr(0, 8) + "_invoice.ps";
	std::cout << "Saving publfile: " << pubFilename << std::endl;
	fileWrite(pubFilename, psPublic);
	
}












void generateWifFromMiniPrivKey(const std::vector<std::string>& args, /* int argc, char* argv[], */ const std::string& thetime)
{		
	const bool compressed(false);	
	
	if(args.size() < 2) //argc < 3)
	{
		std::cout << "Usage: bitgen miniPrivKey MiniPrivateKey" << std::endl;
		throw std::string("Missing argument");
	}
		
	std::cout << "Generating bitcoin address from mini private key" << std::endl;

	std::string miniAddr(args[1]); //argv[2]);
	if(miniAddr == "-")
	{
		std::cout << "Enter the wif value at one row" << std::endl;
		std::cout << "Press enter when all chars are entered" << std::endl;
		
		miniAddr = readStdinLine(30, false, isBase58Char);
	}
	else
	{
		std::cout << "WARNING: The information entered at the command line" << std::endl;
		std::cout << "WARNING: might be stored in a shell history file." << std::endl;
		std::cout << "WARNING: Consider using the \"-\" option for entering information." << std::endl;			
		std::cout << std::endl;		
	}
	
	
	std::cout << "Will convert mini address: " << miniAddr << std::endl;

	if(!isValidMiniPriv(miniAddr))
	{
		throw std::string("Incorrect mini addr");
	}

	//We now know the format is OK. Perform the conversion to standard key
	const BitcoinKeyPair keypair = convertMiniPriv(miniAddr, compressed);
		
		
	std::list<ExtraInfo> generationExtra;
	const ExtraInfo extraInfo("Mini private key", miniAddr);
	generationExtra.push_back(extraInfo);	

	const PrivKey pk = keypair.privaddr.getPriv();
	const Mnemonic myMnemonicObj(pk.key);


	std::string accountName;
	
	const GenerationInfoBase infoBase(thetime, getNodeName(), applicationName, versionNumber);
	
	GenerationParameters params(compressed, true, true);
	
	const int newNum = GenerationMethods::getNextNumberIfConfigured(params);
	
	
	const GenerationInfoCore infoCore(infoBase, newNum, params);
	const GenerationInfo info(infoCore, generationExtra, "Mini private key");
	const AddressGenerationInfo allInfo(keypair.pubaddr.toString(), keypair.privaddr.toString(), "Wif", info);
		
	GenerationMethods::saveAllFiles(allInfo);
}
		


